@@ -650,13 +650,10 @@ class DarkPrep():
         return linDarkobj
 
     @logging_functions.log_fail
-    def prepare(self, params=None):
+    def prepare(self):
         """MAIN FUNCTION"""
         # Read in the yaml parameter file
-        if params is not None:
-            self.params = params
-        else:
-            self.read_parameter_file()
+        self.read_parameter_file()
 
         # Get the log caught up on what's already happened
         self.logger.info('\n\nRunning dark_prep..\n')
