@@ -342,7 +342,7 @@ def make_frame(psfs):
     return frame[:, 38:-38]
 
 
-def psf_lightcurve(psf, ld_coeffs, rp, time, tmodel, plot=False):
+def psf_lightcurve(psf, tmodel=None, time=None):
     """
     Generate a lightcurve for a (76, 76) psf of a given wavelength
 
@@ -350,16 +350,10 @@ def psf_lightcurve(psf, ld_coeffs, rp, time, tmodel, plot=False):
     ----------
     psf: sequencs
         The flux-scaled psf for the given wavelength
-    ld_coeffs: sequence
-        The limb darkening coefficients to use
-    rp: float
-        The planet radius
-    time: sequence
-        The time axis for the TSO
     tmodel: batman.transitmodel.TransitModel
         The transit model of the planet
-    plot: bool
-        Plot the lightcurve
+    time: sequence
+        The time axis for the TSO
 
     Returns
     -------
@@ -373,7 +367,7 @@ def psf_lightcurve(psf, ld_coeffs, rp, time, tmodel, plot=False):
     from mirage.psf import soss_trace as st
     psf = np.ones((76, 76))
     time = np.linspace(-0.2, 0.2, 200)
-    lc = st.psf_lightcurve(psf, None, None, time, None, plot=True)
+    lc = st.psf_lightcurve(psf, time)
 
     Example 2
     ---------
@@ -389,23 +383,19 @@ def psf_lightcurve(psf, ld_coeffs, rp, time, tmodel, plot=False):
     params.inc = 89.8                             # orbital inclination (in degrees)
     params.ecc = 0.                               # eccentricity
     params.w = 90.                                # longitude of periastron (in degrees)
-    params.teff = 3500                            # effective temperature of the host star
-    params.logg = 5                               # log surface gravity of the host star
-    params.feh = 0                                # metallicity of the host star
     params.limb_dark = 'quadratic'                # limb darkening profile to use
     params.u = [1, 1]                             # limb darkening coefficients
+    params.rp = 0.15                              # radius of the planet
+    psf = np.ones((76, 76))
+    time = np.linspace(-0.2, 0.2, 200)
     tmodel = batman.TransitModel(params, time)
-    lc = st.psf_lightcurve(psf, [0.1, 0.1], 0.05, time, tmodel, plot=True)
+    lc = st.psf_lightcurve(psf, tmodel, time)
     """
     # Expand to shape of time axis
     flux = np.tile(psf, (len(time), 1, 1))
 
     # If there is a transiting planet...
-    if ld_coeffs is not None and rp is not None and str(type(tmodel)) == "<class 'batman.transitmodel.TransitModel'>":
-
-        # Set the wavelength dependent orbital parameters
-        tmodel.u = ld_coeffs
-        tmodel.rp = rp
+    if tmodel is not None:
 
         # Generate the light curve for this pixel
         lightcurve = tmodel.light_curve(tmodel)
